Subject: [PATCH 00/00] dm io: delay dec_count
Cc: Milan Broz <mbroz@redhat.com>, Heinz Mauelshagen <hjm@redhat.com>

From: Heinz Mauelshagen <hjm@redhat.com>

Delay decrementing the 'struct io' reference count until after the bio has
been freed so that a bio destructor function may reference it.
Required by a later patch.

Signed-off-by: Heinz Mauelshagen <hjm@redhat.com>
Signed-off-by: Alasdair G Kergon <agk@redhat.com>
Cc: Milan Broz <mbroz@redhat.com>

 drivers/md/dm-io.c |   12 ++++++++++--
 1 files changed, 10 insertions(+), 2 deletions(-)

Index: linux-2.6.19/drivers/md/dm-io.c
===================================================================
--- linux-2.6.19.orig/drivers/md/dm-io.c	2006-12-06 20:49:29.000000000 +0000
+++ linux-2.6.19/drivers/md/dm-io.c	2006-12-06 20:49:37.000000000 +0000
@@ -126,7 +126,8 @@ static void dec_count(struct io *io, uns
 
 static int endio(struct bio *bio, unsigned int done, int error)
 {
-	struct io *io = (struct io *) bio->bi_private;
+	struct io *io;
+	unsigned region;
 
 	/* keep going until we've finished */
 	if (bio->bi_size)
@@ -135,10 +136,17 @@ static int endio(struct bio *bio, unsign
 	if (error && bio_data_dir(bio) == READ)
 		zero_fill_bio(bio);
 
-	dec_count(io, bio_get_region(bio), error);
+	/*
+	 * The bio destructor in bio_put() may use the io object.
+	 */
+	io = bio->bi_private;
+	region = bio_get_region(bio);
+
 	bio->bi_max_vecs++;
 	bio_put(bio);
 
+	dec_count(io, region, error);
+
 	return 0;
 }
 
